#!/usr/bin/env python
# -*- coding:utf-8 -*-
faker_config = {
    'int': {
        'date_time_this_month': [
            'date', 'time'
        ]
    },
    'unicode': {
        'country_code': [
            'country_code'
        ],
        'address': [
            'address', 'street'
        ],
        'postcode': [
            'postcode'
        ],
        'zipcode': [
            'zipcode'
        ],
        'city': [
            'city'
        ],
        'country': [
            'country'
        ],
        'ean8': [
            'code', '_no', '_number'
        ],
        'company': [
            'company'
        ],
        'currency_code': [
            'currency'
        ],
        'free_email': [
            'email'
        ],
        'url': [
            'url', 'attachment'
        ],
        'job': [
            'job', 'role'
        ],
        'text': [
            'text', 'note'
        ],
        'name': [
            'name'
        ],
        'phone_number': [
            'phone'
        ],
        'uuid': [
            'uuid'
        ],
        'object_id': [
            'id'
        ]
    }
}
